==> music_notes-0.10.1.tar.gz: lib/src/harmony/chord.dart <==
part of '../../music_notes.dart';

@immutable
class Chord<T extends Scalable<T>>
    with Chordable<Chord<T>>
    implements Transposable<Chord<T>> {
  /// The [Scalable<T>] items this [Chord<T>] is built of.
  final List<T> items;

  /// Creates a new [Chord<T>] from [items].
  const Chord(this.items);

  /// The root [Scalable<T>] of this [Chord<T>].
  T get root => items.first;

  /// Returns the [ChordPattern] for this [Chord<T>].
  ///
  /// The pattern is calculated based on the intervals between the notes rather
  /// than from the root note. This approach helps differentiate compound
  /// intervals (e.g., [Interval.M9]) from simple intervals (e.g.,
  /// [Interval.M2]) in chords where distance is not explicit (e.g.,
  /// [Note] based chords rather than [PositionedNote] based).
  ///
  /// Example:
  /// ```dart
  /// const Chord([Note.a, Note.c, Note.e]).pattern == ChordPattern.minorTriad
  /// const Chord([Note.g, Note.b, Note.d, Note.f, Note.a]).pattern
  ///   == ChordPattern.majorTriad.add7().add9()
  /// ```
  ChordPattern get pattern => ChordPattern.intervalSteps(items.intervals);

  /// Returns the list of modifier [T]s from the root note.
  ///
  /// Example:
  /// ```dart
  /// Note.a.majorTriad.add7().add9().modifiers == const [Note.g, Note.b]
  /// ```
  List<T> get modifiers => items.skip(3).toList();

  /// Returns a new [Chord<T>] with an [ImperfectQuality.augmented] root
  /// triad.
  ///
  /// Example:
  /// ```dart
  /// Note.c.majorTriad.add7().augmented
  ///   == Chord([Note.c, Note.e, Note.g.sharp, Note.b.flat])
  /// ```
  @override
  Chord<T> get augmented =>
      Chord([...ChordPattern.augmentedTriad.on(root).items, ...modifiers]);

  /// Returns a new [Chord<T>] with an [ImperfectQuality.major] root triad.
  ///
  /// Example:
  /// ```dart
  /// Note.c.minorTriad.add7().major
  ///   == Chord([Note.c, Note.e, Note.g, Note.b.flat])
  /// ```
  @override
  Chord<T> get major =>
      Chord([...ChordPattern.majorTriad.on(root).items, ...modifiers]);

  /// Returns a new [Chord<T>] with an [ImperfectQuality.minor] root triad.
  ///
  /// Example:
  /// ```dart
  /// Note.c.majorTriad.add7().minor
  ///   == Chord([Note.c, Note.e.flat, Note.g, Note.b.flat])
  /// ```
  @override
  Chord<T> get minor =>
      Chord([...ChordPattern.minorTriad.on(root).items, ...modifiers]);

  /// Returns a new [Chord<T>] with an [ImperfectQuality.diminished] root triad.
  ///
  /// Example:
  /// ```dart
  /// Note.c.majorTriad.add7().diminished
  ///   == Chord([Note.c, Note.e.flat, Note.g.flat, Note.b.flat])
  /// ```
  @override
  Chord<T> get diminished =>
      Chord([...ChordPattern.diminishedTriad.on(root).items, ...modifiers]);

  /// Returns a new [Chord<T>] adding [interval].
  @override
  Chord<T> add(Interval interval, {List<int>? replaceSizes}) =>
      pattern.add(interval, replaceSizes: replaceSizes).on(root);

  /// Returns a transposed [Chord<T>] by [interval] from this [Chord<T>].
  ///
  /// Example:
  /// ```dart
  /// const Chord([Note.a, Note.c, Note.e]).transposeBy(Interval.m3)
  ///   == const Chord([Note.c, Note.e.flat, Note.g])
  ///
  /// ChordPattern.majorTriad.on(Note.g.inOctave(4))
  ///   .transposeBy(Interval.M3)
  ///     == const Chord([
  ///          Note.b.inOctave(4),
  ///          Note.d.sharp.inOctave(5),
  ///          Note.f.sharp.inOctave(5)
  ///        ])
  /// ```
  @override
  Chord<T> transposeBy(Interval interval) =>
      Chord(items.map((item) => item.transposeBy(interval)).toList());

  @override
  String toString() => '$root ${pattern.abbreviation} (${items.join(' ')})';

  @override
  bool operator ==(Object other) =>
      other is Chord<T> && ListEquality<T>().equals(items, other.items);

  @override
  int get hashCode => Object.hashAll(items);
}

==> music_notes-0.12.0.tar.gz: lib/src/harmony/chord.dart <==
part of '../../music_notes.dart';

/// A musical chord.
@immutable
class Chord<T extends Scalable<T>>
    with Chordable<Chord<T>>
    implements Transposable<Chord<T>> {
  /// The [Scalable<T>] items this [Chord<T>] is built of.
  final List<T> items;

  /// Creates a new [Chord<T>] from [items].
  const Chord(this.items);

  /// The root [Scalable<T>] of this [Chord<T>].
  T get root => items.first;

  /// Returns the [ChordPattern] for this [Chord<T>].
  ///
  /// The pattern is calculated based on the intervals between the notes rather
  /// than from the root note. This approach helps differentiate compound
  /// intervals (e.g., [Interval.M9]) from simple intervals (e.g.,
  /// [Interval.M2]) in chords where distance is not explicit (e.g.,
  /// [Note] based chords rather than [PositionedNote] based).
  ///
  /// Example:
  /// ```dart
  /// const Chord([Note.a, Note.c, Note.e]).pattern == ChordPattern.minorTriad
  /// const Chord([Note.g, Note.b, Note.d, Note.f, Note.a]).pattern
  ///   == ChordPattern.majorTriad.add7().add9()
  /// ```
  ChordPattern get pattern =>
      ChordPattern.fromIntervalSteps(items._intervalSteps);

  /// Returns the list of modifier [T]s from the root note.
  ///
  /// Example:
  /// ```dart
  /// Note.a.majorTriad.add7().add9().modifiers == const [Note.g, Note.b]
  /// ```
  List<T> get modifiers => items.skip(3).toList();

  /// Returns a new [Chord<T>] with an [ImperfectQuality.diminished] root triad.
  ///
  /// Example:
  /// ```dart
  /// Note.c.majorTriad.add7().diminished
  ///   == Chord([Note.c, Note.e.flat, Note.g.flat, Note.b.flat])
  /// ```
  @override
  Chord<T> get diminished => pattern.diminished.on(root);

  /// Returns a new [Chord<T>] with an [ImperfectQuality.minor] root triad.
  ///
  /// Example:
  /// ```dart
  /// Note.c.majorTriad.add7().minor
  ///   == Chord([Note.c, Note.e.flat, Note.g, Note.b.flat])
  /// ```
  @override
  Chord<T> get minor => pattern.minor.on(root);

  /// Returns a new [Chord<T>] with an [ImperfectQuality.major] root triad.
  ///
  /// Example:
  /// ```dart
  /// Note.c.minorTriad.add7().major
  ///   == Chord([Note.c, Note.e, Note.g, Note.b.flat])
  /// ```
  @override
  Chord<T> get major => pattern.major.on(root);

  /// Returns a new [Chord<T>] with an [ImperfectQuality.augmented] root
  /// triad.
  ///
  /// Example:
  /// ```dart
  /// Note.c.majorTriad.add7().augmented
  ///   == Chord([Note.c, Note.e, Note.g.sharp, Note.b.flat])
  /// ```
  @override
  Chord<T> get augmented => pattern.augmented.on(root);

  /// Returns a new [Chord<T>] adding [interval].
  @override
  Chord<T> add(Interval interval, {List<int>? replaceSizes}) =>
      pattern.add(interval, replaceSizes: replaceSizes).on(root);

  /// Returns a transposed [Chord<T>] by [interval] from this [Chord<T>].
  ///
  /// Example:
  /// ```dart
  /// const Chord([Note.a, Note.c, Note.e]).transposeBy(Interval.m3)
  ///   == const Chord([Note.c, Note.e.flat, Note.g])
  ///
  /// ChordPattern.majorTriad.on(Note.g.inOctave(4))
  ///   .transposeBy(Interval.M3)
  ///     == const Chord([
  ///          Note.b.inOctave(4),
  ///          Note.d.sharp.inOctave(5),
  ///          Note.f.sharp.inOctave(5)
  ///        ])
  /// ```
  @override
  Chord<T> transposeBy(Interval interval) =>
      Chord([for (final item in items) item.transposeBy(interval)]);

  @override
  String toString() => '$root ${pattern.abbreviation} (${items.join(' ')})';

  @override
  bool operator ==(Object other) =>
      other is Chord<T> && ListEquality<T>().equals(items, other.items);

  @override
  int get hashCode => Object.hashAll(items);
}
